package util

import (
	"database/sql"
	"fmt"
	"github.com/BlackCarDriver/config"
	"github.com/BlackCarDriver/log"
	"github.com/codebuilder/model"
	_ "github.com/go-sql-driver/mysql"
	"strings"
)

var (
	db          *sql.DB
	goRes       *log.Logger
	tsRes       *log.Logger
	conf        *config.ConfigMachine
	schemeName  string
	queryResult map[string]column
)

type column map[string]string

// 查询schema下所有的表以及表对应的字段、类型信息
var template string = "SELECT TABLE_NAME as tablename, COLUMN_NAME as columnname, DATA_TYPE as type FROM information_schema.`COLUMNS` " +
	" WHERE TABLE_SCHEMA = '?' ORDER BY TABLE_NAME"

var tconf = model.DBConfig{}

func init() {
	queryResult = make(map[string]column)

	//set the file path that result save in
	log.SetLogPath("./")
	goRes = log.NewLogger("Go.txt")
	tsRes = log.NewLogger("TypeScript.txt")

	//get database connect config from config file
	conf, err := config.NewConfig("./")
	if err != nil {
		fmt.Println("无法获取配置文件!!")
		panic(err)
	}
	conf.SetIsStrict(true)
	schemeName, err = conf.GetString("schema")
	conf.GetStruct("database", &tconf)
	path := strings.Join([]string{tconf.UserName, ":", tconf.Password, "@tcp(", tconf.Host, ":", tconf.Port, ")/", tconf.DbName, "?charset=utf8"}, "")
	db, err = sql.Open("mysql", path)
	err = db.Ping()
	if err != nil {
		fmt.Println(err)
	}
	fmt.Println("Database Connect Scuess!")
}

func QueryTables() {
	template = strings.Replace(template, "?", schemeName, 1)
	rows, err := db.Query(template)
	if err != nil {
		fmt.Println(err)
		panic(err)
	}

	fmt.Println("》》............................获取表基本信息开始............................》》")
	for rows.Next() {
		tableName, colName, colType := "", "", ""
		rows.Scan(&tableName, &colName, &colType)
		_, ok := queryResult[tableName]
		if !ok {
			queryResult[tableName] = make(map[string]string)
			fmt.Println("获取表", tableName, "基本信息已完成.")
		}
		queryResult[tableName][colName] = colType
	}
}

func Report() {
	fmt.Println("》》............................映射结构体开始............................》》")
	for tableName, ctMap := range queryResult {
		structName := CamelCase(tableName)
		goRes.Write("\ntype %s struct {\n", structName)
		tsRes.Write("\nexport class %s {\n", tableName)
		record := "type "
		record += structName
		record += "\t struct {\n"
		for colName, coltype := range ctMap {
			//translate to go struct foramt
			retype := MGMap[coltype]
			if retype == "" {
				retype = coltype
			}
			jsonName := strings.ToLower(colName)
			vname := strings.ToUpper(jsonName[0:1]) + jsonName[1:]
			vname = CamelCase(vname)
			goRes.Write("\t%-10s %-10s `json:\"%s\"`\n", vname, retype, jsonName)
			record += "\n"
			record += vname
			record += "\t"
			record += retype
			//translate to typescript format
			retype = MTMap[coltype]
			if retype == "" {
				retype = coltype
			}
			tsRes.Write("\t%s:%s; \n", jsonName, retype)
		}
		goRes.Write("}\n")
		tsRes.Write("}\n")
		record += "\n}"
		//FlushRecordToRedis(tableName, record)
		fmt.Println("表", tableName, "映射结构体已完成.")
	}
}

func CamelCase(field string) string {
	var ret string
	if strings.Contains(field, "_") {
		arr := strings.Split(field, "_")
		for str := range arr {
			ret += strings.ToUpper(arr[str][0:1]) + arr[str][1:]
		}
	} else {
		return strings.ToUpper(field[0:1]) + field[1:]
	}

	return ret
}
